package de.saring.afw;

import javax.swing.JFrame;

/**
 * This is the main class of the Application FrameWork. Every application that
 * uses this framework must have a main class with this superclass.<br>
 * It contains static references to the document, the view classes of your
 * application. This makes possible that you can e.g. read document data for
 * displaying in your view class.<br>
 * There are also static references to a class which reads the resources of your
 * application.
 *
 * @author  Stefan Saring
 * @version 1.1
 */
public class AFWApplication extends Object
{
    /**
     * The static reference to the document class of the application.
     */
    protected static AFWDocument document;

    /**
     * The static reference to the view class of the application.
     */
    protected static AFWView view;

    /**
     * The static reference to the JFrame, where the view is embedded.
     */
    protected static JFrame frame;

    /**
     * The static reference to the resouce class of the application.
     */
    protected static AFWResourceReader resourceReader;

    /**
     * This flag you should use for printing debug messages. Messages were
     * only shown, when this flag is true.
     */
    protected static boolean debug =false;


    /**
     * Creates a new AFWApplication. The constructor creates also the static
     * instance of AFWResourceReader and the static frame.
     *
     * @param fDebug Only if this flag is true, the application will display
     *        debug messages.
     */
    public AFWApplication (boolean fDebug)
    {
        resourceReader = new AFWResourceReader ();
        frame = new JFrame ();
        this.debug = fDebug;
    }

    /**
     * Makes the application visible. You must construct the view and the
     * document class first, before you call this method.
     *
     * @param xRes The width of the application frame in pixel (-1 for automatic sizing).
     * @param yRes The height of the application frame in pixel (-1 for automatic sizing).
     * @param fResizable Sets the frame window resizable or not.
     * @return True if the application was displayed succesfully.
     */
    protected boolean displayApplication (int xRes, int yRes, boolean fResizable)
    {
        if ( (view == null) || (document == null))
        return false;

        frame.getContentPane ().add (view);
        frame.pack ();
        frame.setResizable (fResizable);

        if ( (xRes >= 0) && (yRes >= 0)) {
            frame.setSize (xRes, yRes);
        }

        frame.setVisible (true);
        return true;
    }


    /**
     * Returns the static document instance of the application.
     *
     * @return The document instance.
     */
    public static AFWDocument getDocument ()
    {
        return document;
    }

    /**
     * Returns the static view instance of the application.
     *
     * @return The view instance.
     */
    public static AFWView getView ()
    {
        return view;
    }

    /**
     * Returns the reference to the JFrame instance, where the view is embedded in.
     *
     * @return The JFrame instance.
     */
    public static JFrame getFrame ()
    {
        return frame;
    }

    /**
     * Returns the static ResourceReader instance of the application.
     *
     * @return The ResourceReader instance.
     */
    public static AFWResourceReader getResReader ()
    {
        return resourceReader;
    }

    /**
     * Returns the debug flag of the application.
     *
     * @return The debug flag.
     */
    public static boolean isDebug ()
    {
        return debug;
    }
}
