package de.saring.jcdiary;

import java.io.File;
import java.util.Locale;
import java.util.prefs.Preferences;
import de.saring.afw.AFWApplication;

/**
 * JCDApp is the main-class of this application. It creates the document-
 * (JCDDocument), the view- (JCDView) and a control-objects (JCDMouseControl).
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class JCDApp extends AFWApplication
{
    /** the preference object of the application */
    private static Preferences preferences = Preferences.userNodeForPackage (JCDApp.class);        
        
    /** the mouse-event listener of the application */
    private static JCDMouseControl mouseControl;

    /** 
     * Constructor for the jcdiary application. 
     * @param debug when true, some information messages will be printed to console
     */
    public JCDApp (boolean debug)
    {
        super (debug);
        checkApplicationDataDir ();        
        
        // set "Kunststoff Look&Feel"
        try {
            javax.swing.UIManager.setLookAndFeel (new com.incors.plaf.kunststoff.KunststoffLookAndFeel ());
        }
        catch (javax.swing.UnsupportedLookAndFeelException e) {}
	
        // use the dafault locale for this system
        Locale.setDefault (Locale.getDefault ());

        // read the application's resource-bundle
        if (resourceReader.readResourceBundle ("resource/i18n/jcdiary-gui", Locale.getDefault ()) == false)
            System.exit (1);
        
        // create the MVC-classes
        document = new JCDDocument ();
        mouseControl = new JCDMouseControl ();
        view = new JCDView ();

        // set the frame properties and make the application visible
        frame.setLocation (preferences.getInt ("mainwindow.x", 10), preferences.getInt ("mainwindow.y", 10));
        displayApplication (preferences.getInt ("mainwindow.width", 400), preferences.getInt ("mainwindow.height", 400), true);

        // read tree from xml-file
        ((JCDDocument) document).readXMLFile ();
    }
    
    /** 
     * Returns the preferences object. 
     * @return the one and only Preferences object
     */
    public static Preferences getPreferences ()
    {
        return preferences;
    }

    /** 
     * Returns the mousecontrol object. 
     * @return the one and only JCDMouseControl object
     */
    public static JCDMouseControl getMouseControl ()
    {
        return mouseControl;
    }

    /**
     * Check if the application data directory allready exists, otherwise try
     * to create it. If this is not possible, the application will be quitted.
     */
    private void checkApplicationDataDir ()
    {
        try {
            File fAppDataDirectory = new File (JCDDocument.JCDIARY_DATA_DIRECTORY);
            if (fAppDataDirectory.exists () == false) 
            {
                // dir does not exist yet => try to create it
                if (fAppDataDirectory.mkdir () == false) {
                    throw new Exception ();
                }
            }
            else {
                // file with same name exits => try to delete file and create dir
                if (fAppDataDirectory.isDirectory () == false) {
                    fAppDataDirectory.delete ();
                    if (fAppDataDirectory.mkdir () == false) {
                        throw new Exception ();
                    }
                }
            }
        }
        catch (Exception ie) {
            System.err.println ("Failed to create the application data directory '" + JCDDocument.JCDIARY_DATA_DIRECTORY + "', please check it...");
            System.exit (-1);
        }
    }
    
    /**
     * Main method for starting the jcdiary application. 
     * @param args application parameters, the first can be "debug" to see some info messages
     */
    public static void main (String args[]) 
    {
        new JCDApp ((args.length > 0) && (args[0].equals ("debug")));
    }
}

