package de.saring.jcdiary;

import java.awt.Cursor;
import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import java.io.*;
import de.saring.afw.AFWDocument;
import de.saring.jcdiary.tree.*;
import de.saring.jcdiary.dialogs.ExportDialog;
import de.saring.jcdiary.xml.*;

/**
* JCDDocument is the model-class of this application. The whole functionality for
* manipulating the tree-structure is implemented in this class.
*
* @author Stefan Saring
* @version 1.0
*/
public class JCDDocument extends AFWDocument
{
    /** The name of the directory where the application data is beeing stored. */
    public static final String JCDIARY_DATA_DIRECTORY = System.getProperty ("user.home") + File.separator + ".jcdiary";
    /** The full filename of the XML file containg the application data. */
    public static final String JCDIARY_XML_FILENAME = JCDIARY_DATA_DIRECTORY + File.separator + "cldiary.xml";
    /** The full filename of the DTD file defining the structure of the application data. */
    public static final String JCDIARY_DTD_FILENAME = JCDIARY_DATA_DIRECTORY + File.separator + "cldiary.dtd";
    
    /** The model of tree in GUI. */
    public DefaultTreeModel treeModel;
    /** The root node of the tree model. */
    public  JCDTreeNode rootNode;
    /** Flag, which will be true, as soon some data has been changed. */
    public  boolean fAnyChanges = false;

    /** The last text entered in search dialog (for proceed searching). */
    public  String lastSearchText;
    /** Flag is true, when comments need to be scanned too on searching. */
    public  boolean fSearchInComments = false;
    /** Flag is set to true, when a search needs to start from the selected node on. */
    public  boolean fStartFromSelection = false;

    /** 
     * Constructs a new JCDDocument.
     */
    public JCDDocument ()
    {
        super ();
        buildNewTree ();
        lastSearchText = new String ("");        
    }

    /**
     * Creates a new tree model.
     */
    private void buildNewTree ()
    {
        DiaryItem diTemp = new DiaryItem (-1,
            JCDApp.getResReader ().getResourceString ("tree.top"), "", false, -1, true,"");
        rootNode = new JCDTreeNode ();
        rootNode.setDiaryItem (diTemp);
        treeModel = new DefaultTreeModel (rootNode);
    }

    /** ***** The next are file operations. ******* */

    /**
     * Reads the tree model from the XML file.
     */
    public void readXMLFile ()
    {
        // does the XML file exists ?
        File fXMLFile = new File (JCDIARY_XML_FILENAME);
        if (fXMLFile.exists () == false) {
            JOptionPane.showMessageDialog (JCDApp.getFrame (),
                JCDApp.getResReader ().getResourceString ("mbox.load_nofile.text"),
                JCDApp.getResReader ().getResourceString ("mbox.load_nofile.title"),
                JOptionPane.INFORMATION_MESSAGE);
            return;
        }

        JCDApp.getFrame ().setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
        JDOMTreeReader xtr = new JDOMTreeReader ();
        boolean ok = xtr.readXMLTree ();
        JCDApp.getFrame ().setCursor (Cursor.getDefaultCursor ());
        
        if (ok == true) 
        {
            // on success set model to view
            ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
            // expand root node (not very elegant :)
            ((JCDView) JCDApp.getView ()).tree.collapseRow (0);
            ((JCDView) JCDApp.getView ()).tree.expandRow (0);
        }
        else {
            JOptionPane.showMessageDialog (JCDApp.getFrame (),
                JCDApp.getResReader ().getResourceString ("mbox.load_error.text"),
                JCDApp.getResReader ().getResourceString ("mbox.load_error.title"),
                JOptionPane.ERROR_MESSAGE);
        }
    }
    
    /**
     * Writes the tree model data to the XML file.
     */
    public boolean writeXMLFile ()
    {
        if (fAnyChanges == false)
            return true;

        if (JOptionPane.showConfirmDialog (JCDApp.getFrame (),
            JCDApp.getResReader ().getResourceString ("mbox.save_changes.text"),
            JCDApp.getResReader ().getResourceString ("mbox.save_changes.title"),
            JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
            == JOptionPane.YES_OPTION)
        {
            JCDApp.getFrame ().setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));

            // first copy DTD and then save diary data           
            boolean ok = copyDTDToApplicationDir ();
            if (ok == true) {
                JDOMTreeWriter xtw = new JDOMTreeWriter ();
                ok = xtw.writeXMLTree ();
                JCDApp.getFrame ().setCursor (Cursor.getDefaultCursor ());                    
            }

            if (ok == false) {
                JOptionPane.showMessageDialog (JCDApp.getFrame (),
                    JCDApp.getResReader ().getResourceString ("mbox.save_error.text"),
                    JCDApp.getResReader ().getResourceString ("mbox.save_error.title"),
                    JOptionPane.ERROR_MESSAGE);
                return false;
            }
        }
        return true;
    }

    /**
     * This method copies the DTD file from the JAR file to the application
     * directory, so the user can use it for something. It's done on every save, 
     * because on updates the DTD can change.
     * @return true on success
     */
    private boolean copyDTDToApplicationDir ()
    {
        try {
            InputStream inputStream = getClass ().getResourceAsStream ("/resource/xml/cldiary.dtd");
            BufferedReader reader = new BufferedReader (new InputStreamReader (inputStream));
            String currentLine;
            StringBuffer strDTDBuffer = new StringBuffer (256);

            // read DTD content from JAR
            while ((currentLine = reader.readLine ()) != null) {  
                strDTDBuffer.append (currentLine + "\n");
            }

            // write DTD content to file
            FileWriter fWriter = new FileWriter (JCDDocument.JCDIARY_DTD_FILENAME);
            fWriter.write (strDTDBuffer.toString ());
            fWriter.flush ();
            fWriter.close ();
            return true;
        }
        catch (IOException e) {
            System.err.println ("Error: Failed to copy DTD file to '" + JCDIARY_DTD_FILENAME + "'...");
            return false;
        }
    }
            
    /**
     * Export diary to HTML (start the ExportDialog).
     */
    public void exportDiaryData ()
    {
        ExportDialog ed = new ExportDialog ();
        ed.setVisible (true);
    }

    /** ***** The next are tree model operations. ******* */

    /**
     * Inserts a new area diary item to the current list (in alphabetical order).
     * @param newTreeItem the area diary item to add
     */
    public void insertArea (DiaryItem newTreeItem)
    {
        JCDTreeNode nodeTemp = new JCDTreeNode ();
        nodeTemp.setDiaryItem (newTreeItem);

        int insertIndex = getAlphabeticIndex (rootNode, newTreeItem.name);
        treeModel.insertNodeInto (nodeTemp, rootNode, insertIndex);
        ((JCDView) JCDApp.getView ()).tree.makeVisible (
            new TreePath (nodeTemp.getPath ()));
        ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
        fAnyChanges = true;
    }

    /**
     * Adds a new group diary item to the specified parent node (in alphabetical
     * order).
     * @param newTreeItem the group diary item to add
     * @param parentNode the parentNode to add the new item too
     */
    public void insertGroup (DiaryItem newTreeItem, JCDTreeNode parentNode)
    {
        JCDTreeNode nodeTemp = new JCDTreeNode ();
        nodeTemp.setDiaryItem (newTreeItem);

        int insertIndex = getAlphabeticIndex (parentNode, newTreeItem.name);
        treeModel.insertNodeInto (nodeTemp, parentNode, insertIndex);
        ((JCDView) JCDApp.getView ()).tree.makeVisible (
            new TreePath (nodeTemp.getPath ()));
        ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
        fAnyChanges = true;
    }

    /**
     * Adds a new rock diary item to the specified parent node (in alphabetical
     * order).
     * @param newTreeItem the rock diary item to add
     * @param parentNode the parentNode to add the new item too
     */
    public void insertRock (DiaryItem newTreeItem, JCDTreeNode parentNode)
    {
        JCDTreeNode nodeTemp = new JCDTreeNode ();
        nodeTemp.setDiaryItem (newTreeItem);

        int insertIndex = getAlphabeticIndex (parentNode, newTreeItem.name);
        treeModel.insertNodeInto (nodeTemp, parentNode, insertIndex);
        ((JCDView) JCDApp.getView ()).tree.makeVisible (
            new TreePath (nodeTemp.getPath ()));
        ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
        fAnyChanges = true;
    }

    /**
     * Adds a new route diary item to the specified parent node (in alphabetical
     * order).
     * @param newTreeItem the route diary item to add
     * @param parentNode the parentNode to add the new item too
     */
    public void insertRoute (DiaryItem diTemp, JCDTreeNode parentNode)
    {
        JCDTreeNode nodeTemp = new JCDTreeNode ();
        nodeTemp.setDiaryItem (diTemp);

        int insertIndex = getAlphabeticIndex (parentNode, diTemp.name);
        treeModel.insertNodeInto (nodeTemp, parentNode, insertIndex);
        ((JCDView) JCDApp.getView ()).tree.makeVisible (
            new TreePath (nodeTemp.getPath ()));
        ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
        fAnyChanges = true;
    }

    /**
     * This method returns the index (in alphabetical order) of the specified
     * string, when it should be added to the parentNode.
     */
    private int getAlphabeticIndex (JCDTreeNode parentNode, String string)
    {
        int index;
        int anzahl = parentNode.getChildCount ();

        for (index = 0; index < anzahl; index++)
        {
            JCDTreeNode tempNode = (JCDTreeNode) parentNode.getChildAt (index);
            DiaryItem diTemp = tempNode.getDiaryItem ();

            if (diTemp.name.compareTo (string) >= 0)
                break;
        }

        return index;
    }

    /** 
     * This method deletes the specified tree node from the tree model.
     * @param treeNode node to delete
     */
    public void deleteTreeNode (JCDTreeNode treeNode)
    {
        // do not delete the root node :)
        if (treeNode.equals (rootNode) == true)
            return;

        if (JOptionPane.showConfirmDialog (JCDApp.getFrame (),
            JCDApp.getResReader ().getResourceString ("mbox.delete_entry.text"),
            JCDApp.getResReader ().getResourceString ("mbox.delete_entry.title"),
            JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
        {
            treeModel.removeNodeFromParent (treeNode);
            ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
            fAnyChanges = true;
        }
    }

    /**
     * Returns the number of the specific entries under the passed entry.
     * This method works recursive.
     * @param treeNode the selected tree node for start recursive counting
     * @return int[] with following content:<br>
     *         pos 0: number of areas <br>
     *         pos 1: number of groups <br>
     *         pos 2: number of rocks <br>
     *         pos 3: number of completed routes <br>
     *         pos 4: number of uncompleted routes <br>
     */
    public int[] countEntries (JCDTreeNode treeNode)
    {
        // start with empty results array
        int[] results = new int[5];
        for (int i = 0; i < results.length; i++) {
            results[i] = 0;
        }

        // process each child of this node
        int childCount = treeNode.getChildCount ();
        for (int i = 0; i < childCount; i++) 
        {
            JCDTreeNode tempNode = (JCDTreeNode) treeNode.getChildAt (i);
            DiaryItem diTemp = tempNode.getDiaryItem ();

            // determine entry type and score it
            switch (diTemp.type) 
            {
                case DiaryItem.TYPE_AREA:
                    results[0]++; break;
                case DiaryItem.TYPE_GROUP:
                    results[1]++; break;
                case DiaryItem.TYPE_ROCK:
                    results[2]++; break;
                case DiaryItem.TYPE_ROUTE:
                    if (diTemp.completed == true)
                        results[3]++;
                    else
                        results[4]++;                        
                    break;
            }

            // is this is a parent node (no route) ? 
            if (diTemp.type < DiaryItem.TYPE_ROUTE)
            {
                // count recursive in all childs and add child to parent results
                int[] childResults = this.countEntries (tempNode);
                
                for (int j = 0; j < results.length; j++) {
                    results[j] += childResults[j];
                }                    
            }
        }
    
        return results;
    }

    /**
     * This method appends the source tree node to the destination tree node
     * (on drag&drop operations).
     * @param srcNode the source of the d&d operation
     * @param destNode the destination of the d&d operation
     */
    public void moveDiaryItem (JCDTreeNode srcNode, JCDTreeNode destNode)
    {
        DiaryItem beSrc = srcNode.getDiaryItem ();
        DiaryItem beDest = destNode.getDiaryItem ();

        // get the alphabetical index for the source node in destination
        int insertIndex = getAlphabeticIndex (destNode, beSrc.name);

        // remove the source node from tree model
        treeModel.removeNodeFromParent (srcNode);
        // append source node to destination node
        treeModel.insertNodeInto (srcNode, destNode, insertIndex);

        // apply tree model changes to view
        ((JCDView) JCDApp.getView ()).tree.setModel (treeModel);
        fAnyChanges = true;
    }
}
