package de.saring.jcdiary;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import de.saring.jcdiary.tree.*;

/**
* JCDMouseControl is a control-class of the application, which is responsible for
* catching and reacting on MouseEvents (drag&drop-events and calling of popup-menus).
*
* @author Stefan Saring
* @version 1.0
*/
public class JCDMouseControl extends Object
implements MouseListener, MouseMotionListener
{
    /** The startnode of a drag&drop operation. */
    private JCDTreeNode dragSourceNode;

    /** ***** The next methods are for catching catch mouse-events ***** */

    /**
     * This method will be called, when a mouse button was pressed.
     * @param e the appropriate MouseEvent
     */
    public void mousePressed (MouseEvent e)
    {
        // is the cursor above a diary item => get path, otherwise exit
        TreePath selPath = ((JCDView) JCDApp.getView ()).tree.getPathForLocation (e.getX (), e.getY ());
        if (selPath != null)
        {
            // show popup menu, when right MB is pressed
            if (e.getModifiers () == InputEvent.BUTTON3_MASK) {
                ((JCDView) JCDApp.getView ()).tree.clearSelection ();
                ((JCDView) JCDApp.getView ()).tree.addSelectionPath (selPath);
                ((JCDView) JCDApp.getView ()).showPopupMenu (e.getX (), e.getY ());
                return;
            }

            // is left MB pressed ?
            if (e.getModifiers () == InputEvent.BUTTON1_MASK)
            {
                // remember the start item (if there will be a drag&drop operation)
                JCDTreeNode node = (JCDTreeNode) selPath.getLastPathComponent ();
                DiaryItem diTemp = node.getDiaryItem ();
                if (diTemp != null) {
                    // it's not possible to remove areas
                    if (diTemp.type > 0)
                        dragSourceNode = node;
                }
            }
        }
    }

    /**
     * This method will be called, when a mouse button was released.
     * @param e the appropriate MouseEvent
     */
    public void mouseReleased (MouseEvent e)
    {
        // reset cursor
        ((JCDView) JCDApp.getView ()).tree.setCursor (new Cursor (Cursor.DEFAULT_CURSOR));

        // is there currently a drag&drop operation in progress ?
        if (dragSourceNode == null)
            return;

        TreePath selPath = ((JCDView) JCDApp.getView ()).tree.getPathForLocation (e.getX (), e.getY ());

        if (selPath != null)
        {
            JCDTreeNode node = (JCDTreeNode) selPath.getLastPathComponent ();
            if (node != null)
            {
                if (isValidDragDestination (node) == true) {
                    // move diary item (first ask the user)
                    DiaryItem diTemp = node.getDiaryItem ();
                    ((JCDView) JCDApp.getView ()).moveDiaryItem (dragSourceNode, node);
                }
            }
        }

        dragSourceNode = null;
    }

    /**
     * This method will be called, when a mouse button was clicked.
     * @param e the appropriate MouseEvent
     */
    public void mouseClicked (MouseEvent e)
    {
        // show route properties in Doubleclick
        if (e.getClickCount () == 2)
        {
            TreePath selPath = ((JCDView) JCDApp.getView ()).tree.getPathForLocation (e.getX (), e.getY ());

            if (selPath != null)
            {
                JCDTreeNode node = (JCDTreeNode) selPath.getLastPathComponent ();
                if (node != null)
                {
                    DiaryItem diTemp = node.getDiaryItem ();
                    if (diTemp.type == 3)
                        ((JCDView) JCDApp.getView ()).editItem ();
                }
            }
            dragSourceNode = null;
        }
    }

    /**
     * This method will be called, when the mouse cursor has entered the tree.
     * @param e the appropriate MouseEvent
     */
    public void mouseEntered (MouseEvent e)
    {
        // reset drag&drop stuff, when window was left 
        if (dragSourceNode != null) 
        {
            dragSourceNode = null;
            ((JCDView) JCDApp.getView ()).tree.setCursor (new Cursor (Cursor.DEFAULT_CURSOR));
        }
    }

    /**
     * This method will be called, when the mouse cursor has exited the tree.
     * @param e the appropriate MouseEvent
     */
    public void mouseExited (MouseEvent e)
    {
        // reset drag&drop stuff, when window was left 
        if (dragSourceNode != null) 
        {
            dragSourceNode = null;
            ((JCDView) JCDApp.getView ()).tree.setCursor (new Cursor (Cursor.DEFAULT_CURSOR));
        }
    }

    /** ***** The next methods are for catching catch mouse move-events ***** */

    /**
     * This method will be called, when the mouse cursor has been moved.
     * @param e the appropriate MouseEvent
     */
    public void mouseMoved (MouseEvent e) 
    {
    }

    /**
     * This method will be called, when the mouse cursor has been dragged.
     * @param e the appropriate MouseEvent
     */
    public void mouseDragged (MouseEvent e)
    {
        // is there currently a drag&drop operation in progress ?
        if (dragSourceNode == null)
            return;

        // get the current diary item below the "drag" mouse cursor
        TreePath selPath = ((JCDView) JCDApp.getView ()).tree.getPathForLocation (e.getX (), e.getY ());

        if (selPath != null)
        {
            // select the current diary node below the mouse cursor
            ((JCDView) JCDApp.getView ()).tree.setSelectionPath (selPath);
            JCDTreeNode node = (JCDTreeNode) selPath.getLastPathComponent ();
            if (node != null)
            {
                DiaryItem diTemp = node.getDiaryItem ();
                // display "hand" cursor, if the destination is valid for d&d
                if (isValidDragDestination (node) == true)
                    ((JCDView) JCDApp.getView ()).tree.setCursor (new Cursor (Cursor.HAND_CURSOR));
                else
                    ((JCDView) JCDApp.getView ()).tree.setCursor (new Cursor (Cursor.DEFAULT_CURSOR));
            }
        }
    }

    /**
     * This method checks, whether the specified node is a valid drag&drop
     * operation according to the current d&d start node.
     * @param destNode destination node for drag&drop
     * @return true, when the destination is valid
     */
    private boolean isValidDragDestination (JCDTreeNode destNode)
    {
        DiaryItem nodeSrc = dragSourceNode.getDiaryItem ();
        DiaryItem nodeDest = destNode.getDiaryItem ();

        // not valid, when source and destination are same
        if (dragSourceNode == destNode)
            return false;

        // the destination contains allready the source => not valid
        if (destNode.isNodeChild (dragSourceNode) == true)
            return false;

        switch (nodeSrc.type)
        {
            case DiaryItem.TYPE_GROUP:
                // groups can only be appended to areas or other groups
                if ((nodeDest.type == DiaryItem.TYPE_AREA) || (nodeDest.type == DiaryItem.TYPE_GROUP))
                    return true;
                else
                    return false;
            case DiaryItem.TYPE_ROCK:
                // rocks can only be appended to areas or other groups
                if ((nodeDest.type == DiaryItem.TYPE_AREA) || (nodeDest.type == DiaryItem.TYPE_GROUP))
                    return true;
                else
                    return false;
            case DiaryItem.TYPE_ROUTE:
                // routes can only be appended to rocks
                if (nodeDest.type == DiaryItem.TYPE_ROCK)
                    return true;
                else
                    return false;
        }
        return false;
    }
}

