package de.saring.jcdiary.dialogs;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import de.saring.jcdiary.JCDApp;
import de.saring.jcdiary.tree.DiaryItem;


/**
 * This class implements an dialog for editing group entries in the climbers diary.
 * Group entries are climbing areas, rock groups and rocks. It is possible to edit
 * the name and an comment of an group entry.
 *
 * @author Stefan Saring
 * @version 2.0
 */
public class GroupDialog extends JDialog implements ActionListener
{
    private DiaryItem diaryItem;
    private JButton bOK, bCancel;
    private JTextField tfName;
    private JTextArea taComment;
    private boolean fDialogCanceled = true;

    /**
     * Creates the group dialog for the specified diary item.
     * @param dItem diary item to display
     */
    public GroupDialog (DiaryItem dItem)
    {
        super (JCDApp.getFrame (), true);
        this.diaryItem = dItem;

        // set dialog name
        switch (diaryItem.type) {
            case 0:
                setTitle (JCDApp.getResReader ().getResourceString ("group.area.title"));
                break;
            case 1:
                setTitle (JCDApp.getResReader ().getResourceString ("group.group.title"));
                break;
            case 2:
                setTitle (JCDApp.getResReader ().getResourceString ("group.rock.title"));
                break;
        }

        init_GUI ();
        setResizable (true);
        setLocationRelativeTo (JCDApp.getFrame ());        
        setDiaryItemData ();

        // set window listener (always close)
        addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent e) {
                dispose ();
            }
        });
    }

    /**
     * Constructs GUI of dialog.
     */
    private void init_GUI ()
    {
        JPanel rootPane = new JPanel ();
        rootPane.setLayout (new GridBagLayout ());                
        rootPane.setBorder (new EmptyBorder (10, 10, 10, 10));        

        // create label and textfield for group name
        JLabel lName = new JLabel (JCDApp.getResReader ().getResourceString ("group.name"), JLabel.LEFT);
        tfName = new JTextField ("", 20);
        rootPane.add (lName, new GridBagConstraints (0, 0, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 0, 0, 5), 0, 0));
        rootPane.add (tfName, new GridBagConstraints (1, 0, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets (0, 5, 0, 0), 0, 0));

        // create label for group comment
        JLabel lComment = new JLabel (JCDApp.getResReader ().getResourceString ("group.comment"), JLabel.LEFT);
        rootPane.add (lComment, new GridBagConstraints (0, 1, 2, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (15, 0, 5, 0), 0, 0));
        
        // create textarea for group comment
        taComment = new JTextArea ();
        taComment.setColumns (40);
        taComment.setRows (6);
        taComment.setLineWrap (true);
        taComment.setWrapStyleWord (true);
        JScrollPane spComment = new JScrollPane ();
        spComment.getViewport ().add (taComment);
        rootPane.add (spComment, new GridBagConstraints (0, 2, 2, 1, 0f, 0f, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets (0, 0, 0, 0), 0, 0));

        // create OK and Cancel buttons
        bOK = new JButton (JCDApp.getResReader ().getResourceString ("group.ok"));
        bOK.addActionListener (this);
        bCancel = new JButton (JCDApp.getResReader ().getResourceString ("group.cancel"));
        bCancel.addActionListener (this);
        
        FlowLayout flButtons = new FlowLayout ();
        flButtons.setHgap (10);
        JPanel pButtons = new JPanel (flButtons);
        pButtons.add (bOK);
        pButtons.add (bCancel);
        rootPane.add (pButtons, new GridBagConstraints (0, 3, 2, 1, 0f, 0f, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets (10, 0, 0, 0), 0, 0));

        // add root pane 
        getContentPane ().add (rootPane);
        pack ();
    }

    /**
     * Fill GUI controls with diary item data.
     */
    private void setDiaryItemData ()
    {
        tfName.setText (diaryItem.name);
        taComment.setText (diaryItem.comment);
    }

    /**
     * Fill diary item object with GUI control values.
     */
    public void storeInputData ()
    {
        // is there a name ?
        if (tfName.getText ().length () != 0)
            diaryItem.name = tfName.getText ();
        else {
            displayError (JCDApp.getResReader ().getResourceString ("group.error.noname"));
            return;
        }

        // take comment
        diaryItem.comment = taComment.getText ();

        // dialog was closed with OK button
        fDialogCanceled = false;
        dispose ();
    }

    /**
     * Displays the appropriate error message.
     * @param sError message to display
     */
    public void displayError (String sError)
    {
        JOptionPane.showMessageDialog (JCDApp.getFrame (), sError,
            JCDApp.getResReader ().getResourceString ("group.error.title"),
            JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Handle action events.
     * @param e the actionevent to handle
     */
    public void actionPerformed (ActionEvent e)
    {
        Object source = e.getSource();

        if (source == bOK) {
            storeInputData ();
        }

        if (source == bCancel) {
            dispose ();
        }
    }
    
    /**
     * Returns the modified diary item.
     * @return the modifed item
     */
    public DiaryItem getDiaryItem ()
    {
        return diaryItem;
    }
    
    /**
     * Returns true, when the dialog was canceled Fales when OK button was pressed.
     * return the close mode
     */
    public boolean isDialogCanceled ()
    {
        return fDialogCanceled;
    }
}
