package de.saring.jcdiary.export;

import java.io.*;
import java.net.URL;
import java.text.DateFormat;
import java.util.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import de.saring.afw.AFWResourceReader;
import de.saring.jcdiary.JCDApp;
import de.saring.jcdiary.tree.JCDTreeNode;
import de.saring.jcdiary.xml.JDOMTreeWriter;
import org.jdom.*;
import org.jdom.output.DOMOutputter;

/**
 * This class exports the diary to by creating a XML data stream for the diary 
 * items to export. This stream can be transformed by a XSL stylesheet to any 
 * target file format (e.g. HTML). THE XSL transformation is done by using the
 * Xalan library (part of JDK since 1.4).
 *
 * @author  Stefan Saring
 * @version 1.0
 */
public class XSLDiaryExporter
{    
    /** Start and end tag for i18n string replacements in the XSL result string. */
    private static final String TAG_I18N_START = "${$";
    private static final String TAG_I18N_END   = "$}$";

    /** Start and end tag for command string replacements in the XSL result string. */
    private static final String TAG_COMMAND_START = "$($";
    private static final String TAG_COMMAND_END   = "$)$";

    /** Replacement command names for the XSL result. */
    private static final String COMMAND_EXPORT_DATE = "export.date";
    
    /** This W3C DOM object contains the XML data of the diary items to export. */
    private org.w3c.dom.Document w3cDocument;

    
    /** 
     * Creates a new instance of XSLDiaryExporter.
     * @param rootTreeNode4Export root TreeNode containing all diary data to export
     */
    public XSLDiaryExporter (JCDTreeNode rootTreeNode4Export) throws Exception
    {
        // get JDOM document for the tree data which has to be exported
        JDOMTreeWriter jdomWriter = new JDOMTreeWriter ();
        org.jdom.Document jdomDocument = jdomWriter.createJDOMDocumentForDiaryTree (rootTreeNode4Export);

        // create w3c document of JDOM document for XSL transformation
        DOMOutputter domOutputter = new DOMOutputter ();
        try {
            this.w3cDocument = domOutputter.output (jdomDocument);
        }
        catch (JDOMException je) {
            System.err.println ("Exception" + je.getMessage ());
            throw new Exception ("Failed to create W3C DOM document from JDOM document ...");
        }
    }
    
    /**
     * This method exports the XML diary data by using the specified stylesheet
     * to the specified target file.
     * @param stylesheet URL of stylesheet to use (URL because they can be in JAR files)
     * @param target filename for storing the export result
     */
    public void exportDiary (URL stylesheetURL, String target) throws Exception
    {              
        if (stylesheetURL == null) {
            throw new Exception ("XSL Stylesheet URL is null ...");
        }
        
    	// create XSL Transformer
        Transformer transformer = null;
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance ();        
            transformer = tFactory.newTransformer (new StreamSource (stylesheetURL.toString ()));
        }
        catch (TransformerConfigurationException tce) {
            System.err.println ("Exception" + tce.getMessage ());
            throw new Exception ("Failed to configure Transformer for XSL transformation...");
        }
        
        // start transformation
        StringWriter strWriterResult = new StringWriter ();
        try {
            transformer.transform (new DOMSource (this.w3cDocument), new StreamResult (strWriterResult));
        }
        catch (TransformerException te) {
            System.err.println ("Exception" + te.getMessage ());
            throw new Exception ("Failed to transform the XML diary data to target format...");
        }
             
        String strXSLResult = strWriterResult.toString ();        
        try {            
            // replace resource ID's in XSL result with resource values
            strXSLResult = replaceI18NTags (strXSLResult);
            // replace command names in XSL result with command values
            strXSLResult = replaceCommandTags (strXSLResult);
        }
        catch (Exception e) {
            System.err.println ("Exception" + e.getMessage ());
            throw new Exception ("Failed to replace i18n string and commands in the generated XSL string...");
        }
  
        // finally write export result to file
        try {
            FileOutputStream fos = new FileOutputStream (target);
            fos.write (strXSLResult.getBytes ());
            fos.flush ();
            fos.close ();
        }
        catch (IOException ie) {
            System.err.println ("Exception" + ie.getMessage ());
            throw new Exception ("Failed to write the export result to the specified file...");
        }
    }
    
    /**
     * This method finds all i18n-tags in the XSL result string and replaces them
     * with the appropriate value of the resource ID between the i18n-tags.
     * @param strXSLResult the XSL result string
     * @return the modified XSL result string
     */
    private String replaceI18NTags (String strXSLResult) throws Exception
    {
        // create a new String based on the XSL result
        AFWResourceReader resourceReader = JCDApp.getResReader ();
        StringBuffer strBufferI18N = new StringBuffer (2048);
        int nextStartTagPos = 0;
        int currentPos = 0;
        
        // find next i18n-tag
        while ((nextStartTagPos = strXSLResult.indexOf (TAG_I18N_START, currentPos)) != -1)
        {
            // append substring from current position until this tag to the new StringBuffer
            strBufferI18N.append (strXSLResult.substring (currentPos, nextStartTagPos));
            
            // find end tag for current tag
            int endTagPos = strXSLResult.indexOf (TAG_I18N_END, nextStartTagPos + TAG_I18N_START.length ());
            if (endTagPos == -1) {
                throw new Exception ("There's no i18n-end-tag in the XSL result string...");
            }
            
            // get resource ID between start and end tag
            String strResourceID = strXSLResult.substring (nextStartTagPos + TAG_I18N_START.length (), endTagPos);            
            strBufferI18N.append (resourceReader.getResourceString (strResourceID));

            // continue search for next tag after this tag
            currentPos = endTagPos + TAG_I18N_END.length ();
        }
        
        // no more tags => append substring from current position until end to the new StringBuffer 
        strBufferI18N.append (strXSLResult.substring (currentPos));
        return strBufferI18N.toString ();
    }

    /**
     * This method finds all command-tags in the XSL result string and replaces them
     * with the appropriate command value (e.g. the export date).
     * @param strXSLResult the XSL result string
     * @return the modified XSL result string
     */
    private String replaceCommandTags (String strXSLResult) throws Exception
    {
        // create a new String based on the XSL result
        StringBuffer strBufferCommand = new StringBuffer (2048);
        int nextStartTagPos = 0;
        int currentPos = 0;
        
        // find next command-tag        
        while ((nextStartTagPos = strXSLResult.indexOf (TAG_COMMAND_START, currentPos)) != -1)
        {
            // append substring from current position until this tag to the new StringBuffer
            strBufferCommand.append (strXSLResult.substring (currentPos, nextStartTagPos));
            
            // find end tag for current tag
            int endTagPos = strXSLResult.indexOf (TAG_COMMAND_END, nextStartTagPos + TAG_COMMAND_START.length ());
            if (endTagPos == -1) {
                throw new Exception ("There's no command-end-tag in the XSL result string...");
            }
            
            // get command name 
            String strCommandName = strXSLResult.substring (nextStartTagPos + TAG_COMMAND_START.length (), endTagPos);            
            // append command value to new StringBuffer
            strBufferCommand.append (getCommandValue (strCommandName));

            // continue search for next tag after this tag
            currentPos = endTagPos + TAG_I18N_END.length ();
        }
        
        // no more tags => append substring from current position until end to the new StringBuffer 
        strBufferCommand.append (strXSLResult.substring (currentPos));
        return strBufferCommand.toString ();
    }
    
    /**
     * This method is the command "dictonary". For all defined command names
     * it returns the appropriate value. If command is not defined "???" will
     * be rerturned.
     * @param commandName name of command
     * @return the command value
     */
    private String getCommandValue (String commandName)
    {
        // return the export (current) date string
        if (commandName.toLowerCase ().equals (COMMAND_EXPORT_DATE)) 
        {
            Calendar calendar = Calendar.getInstance (Locale.getDefault ());
            DateFormat dFormat = DateFormat.getDateInstance (DateFormat.MEDIUM);
            return dFormat.format (calendar.getTime ());            
        }
        
        return "???";
    }
}
