package de.saring.jcdiary.tree;

import javax.swing.tree.*;

/**
* This class enhances the DefaultMutableTreeNode-class for a reference to the
* DiaryItem-object that belongs to this tree node.
*
* @author Stefan Saring
* @version 2.0
*/
public class JCDTreeNode extends DefaultMutableTreeNode
{
    /** The diary item object that belongs to this node. */
    private  DiaryItem diaryItem;
    
    /**
     * Sets the diary item of the tree node.     
     */
    public void setDiaryItem (DiaryItem diTemp)
    {
    	diaryItem = diTemp;
    }

    /**
     * Returns the diary item of the tree node.
     */
    public DiaryItem getDiaryItem ()
    {
        return diaryItem; 
    }
    
    /**
     * The object's string representation is the name of diary item
     * (is shown in the tree).
     */
    public String toString ()
    {
        return diaryItem.name;
    }
    
    /**
     * This method returns a new root node of the tree model, which contains
     * only the parent nodes of this node (in the same hierarchy) including
     * this node with it's full subcontent. The returned node has the same hierarchy
     * structure as the tree model.<br>
     * This method is usefull, when the user want's e.g. export the selected
     * node and all it's subcontent to HTML. So this method returns the complete
     * tree structure to this node without the parent nodes other child node and
     * this node with it's full sub nodes (all the group-, rock- and route-nodes
     + from this node on).
     * @return the root tree node containing only the parent's and this node's content only
     */
    public JCDTreeNode getRootNodeWithThisNodesContentOnly ()
    {
        // when this node is allready the root element => return it
        if (this.isRoot () == true) {
            return this;
        }
        
        // create a new root node
        JCDTreeNode newRootNode = new JCDTreeNode ();
        JCDTreeNode currentNode = newRootNode;
        
        // process all nodes in path to this node
        TreeNode[] pathToRoot = this.getPath ();
        for (int i = 0; i < pathToRoot.length; i++)
        {
            // set diary item of original tree node to the current new node
            currentNode.setDiaryItem (((JCDTreeNode) pathToRoot[i]).getDiaryItem ()); 
            
            // if this is the last node in array => exit loop
            if (i >= pathToRoot.length - 1) {
                break;
            }
            
            // append a new child node to current node and proceed with child node
            JCDTreeNode childNode = new JCDTreeNode ();
            currentNode.add (childNode);
            currentNode = childNode;
        }           

        copyTreeNodeContent (this, currentNode);        
        return newRootNode;
    }
    
    /**
     * This method creates for each child node of orgNode a new child node for
     * newNode and set's the DiaryItem of the orgNode's child to the new created
     * newNode's child.<br>
     * This method is recursive, so it calls this method for each child node
     * of orgNode again. So this newNode is finally a copy of orgNode, with the
     * same DiaryItem objects.
     * @param orgNode source node to copy the childs
     * @param newNode the node, where the new childs have to be appended
     */
    private void copyTreeNodeContent (JCDTreeNode orgNode, JCDTreeNode newNode) 
    {
        for (int i = 0; i < orgNode.getChildCount (); i++)
        {
            JCDTreeNode orgChildNode = (JCDTreeNode) orgNode.getChildAt (i);
            JCDTreeNode newChildNode = new JCDTreeNode ();
            newChildNode.setDiaryItem (orgChildNode.getDiaryItem ());
            newNode.add (newChildNode);
            copyTreeNodeContent (orgChildNode, newChildNode);
        }
    }
}

