package de.saring.jcdiary.xml;

import java.io.*;
import java.util.*;
import org.jdom.*;  
import org.jdom.input.*;
import de.saring.jcdiary.*;
import de.saring.jcdiary.tree.*;

/**
 * The JDOMTreeReader is for reading the data of the climbers diary from a XML-file
 * and for creating the JCDDocument-content. The reading and the converting of the
 * XML data is realized by using the JDOM API.
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class JDOMTreeReader extends Object
{
    /**
     * This method reads the jcdiary application data from the appropriate
     * XML file and constructs the tree model to be shown by the view.
     * @return true in case of success
     */
    public boolean readXMLTree ()
    {
        Document document;
        
        // create a validating SAX parser and read document
        try {            
            SAXBuilder builder = new SAXBuilder (true);
            document = builder.build (new File (JCDDocument.JCDIARY_XML_FILENAME));
        } 
        catch (JDOMException e)
        {
            if (e.getCause () != null)
                e.getCause ().printStackTrace ();
            else
                e.printStackTrace ();
            return false;
        } 
        catch (Exception e) 
        {
            e.printStackTrace ();
            return false;
        }
        
        // get root element and read all the contained area entries
        Element eClimbersdiary = document.getRootElement ();
        List lAreas = eClimbersdiary.getChildren (XMLConstants.ELEMENT_AREA);
        Iterator iAreas = lAreas.iterator ();
        
        while (iAreas.hasNext ()) {            
            readArea ((Element) iAreas.next (), ((JCDDocument) JCDApp.getDocument ()).rootNode);
        }
        
        return true;
    }

    
    /**
     * Create an "area" tree item out of the specified Element and insert it into
     * the passed tree node.
     * 
     * @param eArea JDOM element containing the area info
     * @param parentNode the tree node append the area item too
     */    
    private void readArea (Element eArea, JCDTreeNode parentNode)
    {
        // read attys of area
        String strName = eArea.getChildText (XMLConstants.ELEMENT_ENTRY_NAME);
        String strComment = eArea.getChildText (XMLConstants.ELEMENT_ENTRY_COMMENT);

        // create a new JCDTreeNode and insert it to parent node (alphabetical order)
        JCDTreeNode childNode = new JCDTreeNode ();
        DiaryItem diTemp = new DiaryItem  (DiaryItem.TYPE_AREA, strName, "", false, -1, true, strComment);
        childNode.setDiaryItem (diTemp);
        int lastIndex = ((JCDDocument) JCDApp.getDocument ()).treeModel.getChildCount (parentNode);
        ((JCDDocument) JCDApp.getDocument ()).treeModel.insertNodeInto (childNode, parentNode, lastIndex);

        // a "area" element can contains "group" and "rock" elements
        // => get all "group" children elements and create tree entries for them
        List lGroups = eArea.getChildren (XMLConstants.ELEMENT_GROUP);
        Iterator iGroups = lGroups.iterator ();        
        while (iGroups.hasNext ()) {
            readGroup ((Element) iGroups.next (), childNode);
        }
        
        // => get all "rock" children elements and create tree entries for them
        List lRocks = eArea.getChildren (XMLConstants.ELEMENT_ROCK);
        Iterator iRocks = lRocks.iterator ();        
        while (iRocks.hasNext ()) {
            readRock ((Element) iRocks.next (), childNode);
        }
    }

    /**
     * Create an "group" tree item out of the specified Element and insert it into
     * the passed tree node.
     * 
     * @param eGroup JDOM element containing the group info
     * @param parentNode the tree node append the group item too
     */    
    private void readGroup (Element eGroup, JCDTreeNode parentNode)
    {
        // read attys of group
        String strName = eGroup.getChildText (XMLConstants.ELEMENT_ENTRY_NAME);
        String strComment = eGroup.getChildText (XMLConstants.ELEMENT_ENTRY_COMMENT);
                    
        // create a new JCDTreeNode and insert it to parent node (alphabetical order)
        JCDTreeNode childNode = new JCDTreeNode ();
        DiaryItem diTemp = new DiaryItem  (DiaryItem.TYPE_GROUP, strName, "", false, -1, true, strComment);
        childNode.setDiaryItem (diTemp);
        int lastIndex = ((JCDDocument) JCDApp.getDocument ()).treeModel.getChildCount (parentNode);
        ((JCDDocument) JCDApp.getDocument ()).treeModel.insertNodeInto (childNode, parentNode, lastIndex);

        // a "group" element can contains "group" and "rock" elements
        // => get all "group" children elements and create tree entries for them
        List lGroups = eGroup.getChildren (XMLConstants.ELEMENT_GROUP);
        Iterator iGroups = lGroups.iterator ();        
        while (iGroups.hasNext ()) {
            readGroup ((Element) iGroups.next (), childNode);
        }
        
        // => get all "rock" children elements and create tree entries for them
        List lRocks = eGroup.getChildren (XMLConstants.ELEMENT_ROCK);
        Iterator iRocks = lRocks.iterator ();        
        while (iRocks.hasNext ()) {
            readRock ((Element) iRocks.next (), childNode);
        }
    }
    
    /**
     * Create an "rock" tree item out of the specified Element and insert it into
     * the passed tree node.
     * 
     * @param eRock JDOM element containing the rock info
     * @param parentNode the tree node append the rock item too
     */    
    private void readRock (Element eRock, JCDTreeNode parentNode)
    {
        // read attys of rock
        String strName = eRock.getChildText (XMLConstants.ELEMENT_ENTRY_NAME);
        String strComment = eRock.getChildText (XMLConstants.ELEMENT_ENTRY_COMMENT);
                    
        // create a new JCDTreeNode and insert it to parent node (alphabetical order)
        JCDTreeNode childNode = new JCDTreeNode ();
        DiaryItem diTemp = new DiaryItem  (DiaryItem.TYPE_ROCK, strName, "", false, -1, true, strComment);
        childNode.setDiaryItem (diTemp);
        int lastIndex = ((JCDDocument) JCDApp.getDocument ()).treeModel.getChildCount (parentNode);
        ((JCDDocument) JCDApp.getDocument ()).treeModel.insertNodeInto (childNode, parentNode, lastIndex);

        // => get all "route" children elements and create tree entries for them
        List lRoutes = eRock.getChildren (XMLConstants.ELEMENT_ROUTE);
        Iterator iRoutes = lRoutes.iterator ();        
        while (iRoutes.hasNext ()) {
            readRoute ((Element) iRoutes.next (), childNode);
        }
    }
    
    /**
     * Create an "route" tree item out of the specified Element and insert it into
     * the passed tree node.
     * 
     * @param eRoute JDOM element containing the route info
     * @param parentNode the tree node append the route item too
     */    
    private void readRoute (Element eRoute, JCDTreeNode parentNode)
    {
        boolean fLead = false;
        long lDate = -1;
        boolean fCompleted = true;

        // read attys of route
        String strName =      eRoute.getChildText (XMLConstants.ELEMENT_ENTRY_NAME);
        String strDifficulty = eRoute.getChildText (XMLConstants.ELEMENT_ROUTE_DIFFICULTY);
        String strLead       = eRoute.getChildText (XMLConstants.ELEMENT_ROUTE_LEAD);
        String strDate       = eRoute.getChildText (XMLConstants.ELEMENT_ROUTE_DATE);
        String strComment    = eRoute.getChildText (XMLConstants.ELEMENT_ENTRY_COMMENT);

        // read optional "completed" atty
        String strCompleted = null;
        if (eRoute.getAttribute (XMLConstants.ATTY_ROUTE_COMPLETED) != null) {
            strCompleted = eRoute.getAttribute (XMLConstants.ATTY_ROUTE_COMPLETED).getValue ();
        }

        // convert date
        try {
            lDate = Long.parseLong (strDate);
        }
        catch (NumberFormatException nfe) {
            System.err.println ("Error: Content of XML-tag 'date' is not valid !");
            nfe.printStackTrace ();
        }
        
        // convert lead flag (is defined as boolean in XML Schema Definition
        // => so it can be "0|1" or "false|true")
        if ((strLead.equals ("1")) || (strLead.toLowerCase ().equals ("true"))) {
            fLead = true;
        }
        
        // convert completed flag (is defined as boolean in XML Schema Definition
        // => so it can be "0|1" or "false|true")
        // "true" is default, so check only for "0" or false        
        if ((strCompleted != null) &&
            ((strCompleted.equals ("0")) || (strLead.toLowerCase ().equals ("false")))) 
        {
            fCompleted = false;
        }
        
        // create a new JCDTreeNode and insert it to parent node (alphabetical order)        
        JCDTreeNode childNode = new JCDTreeNode ();
        DiaryItem diTemp = new DiaryItem (DiaryItem.TYPE_ROUTE, strName, strDifficulty, fLead, lDate, fCompleted, strComment);
        childNode.setDiaryItem (diTemp);
        int lastIndex = ((JCDDocument) JCDApp.getDocument ()).treeModel.getChildCount (parentNode);
        ((JCDDocument) JCDApp.getDocument ()).treeModel.insertNodeInto (childNode, parentNode, lastIndex);
    }
}
