package de.saring.jcdiary.xml;

import java.io.*;
import org.jdom.*;  
import org.jdom.output.*;
import de.saring.jcdiary.*;
import de.saring.jcdiary.tree.*;

/**
 * The JDOMTreeWriter class creates a JDOM-Document Object Model (DOM) of the application
 * data and writes it to a XML file
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class JDOMTreeWriter extends Object
{
    /** 
     * Creates a JDOM document from the diary data and stores it in an XML file. 
     */
    public boolean writeXMLTree ()
    {
        // create JDOM document from the tree's root node on
        JCDTreeNode rootNode = ((JCDDocument) JCDApp.getDocument ()).rootNode, rootElement;
        Document document = createJDOMDocumentForDiaryTree (rootNode);

        // set doctype (DTD) of XML data
        document.setDocType (new DocType ("climbersdiary", "cldiary.dtd"));
        
        // write (J)DOM document to XML-file
        try {
            XMLOutputter outputter = new XMLOutputter ("  ", true, "ISO-8859-1");
            FileWriter fWriter = new FileWriter (JCDDocument.JCDIARY_XML_FILENAME);
            outputter.output (document, fWriter);
            fWriter.flush ();
            fWriter.close (); 
        } 
        catch (IOException e) 
        {
            e.printStackTrace();
            return false;
        }
        
	    return true;
    }

    /**
     * This method creates a JDOM XML document containing all the diary data
     * from the tree's root node on.
     * @return the created JDOM document
     */
    public Document createJDOMDocumentForDiaryTree (JCDTreeNode rootNode)
    {
        Element rootElement = new Element ("climbersdiary");
        recursivTreeStore (rootNode, rootElement);
        return new Document (rootElement);
    }
    
    /**
     * Goes recursively through the whole diary tree and creates a JDOM element 
     * for every diary item.
     * @param node the start diary node for recursion
     * @param element the current JDOM element
     */
    private void recursivTreeStore (JCDTreeNode node, Element element)
    {
        DiaryItem diTemp = node.getDiaryItem ();
	
    	if (diTemp.type <= 2)
	        createGroupElement (diTemp, element);
    	else
	        createRouteElement (diTemp, element);

    	for (int i = 0; i < node.getChildCount (); i++) 
	    {
	        JCDTreeNode childNode = (JCDTreeNode) node.getChildAt (i);
    	    String tagName = "";
	        
	        switch (childNode.getDiaryItem ().type) 
    	    {
	    	    case 0: tagName = XMLConstants.ELEMENT_AREA; break;
    		    case 1: tagName = XMLConstants.ELEMENT_GROUP; break;
	        	case 2: tagName = XMLConstants.ELEMENT_ROCK; break;
        		case 3: tagName = XMLConstants.ELEMENT_ROUTE; break;
    	    }

    	    Element childElement = new Element (tagName);
	        element.addContent (childElement);	
	        recursivTreeStore (childNode, childElement);
    	}
    }

    /**
     * Create a "group" child element (area, group or rock) and add it to the specified
     * parent element.
     * @param entry diary entry containing the data
     * @param element the JDOM element add the created child too
     */
    private void createGroupElement (DiaryItem entry, Element element)
    {
	    // ignore the root node
    	if (entry.type < 0)
	        return;

    	// create <name> element and add it
	    Element eName = new Element (XMLConstants.ELEMENT_ENTRY_NAME);
	    eName.setText (entry.name);
    	element.addContent (eName);

    	// create <comment> element and add it
	    Element eComment = new Element (XMLConstants.ELEMENT_ENTRY_COMMENT);
	    eComment.setText (entry.comment);
    	element.addContent (eComment);	
    }

    /**
     * Create a "group" child element (area, group or rock) and add it to the specified
     * parent element.
     * @param entry diary entry containing the data
     * @param element the JDOM element add the created child too
     */
    private void createRouteElement (DiaryItem entry, Element element)
    {
    	// create <name> element and add it
    	Element eName = new Element (XMLConstants.ELEMENT_ENTRY_NAME);
    	eName.setText (entry.name);
    	element.addContent (eName);	

    	// create <difficulty> element and add it
    	Element eDifficulty = new Element (XMLConstants.ELEMENT_ROUTE_DIFFICULTY);
    	eDifficulty.setText (entry.difficulty);
    	element.addContent (eDifficulty);	

    	// create <lead> element and add it
	    Element eLead = new Element (XMLConstants.ELEMENT_ROUTE_LEAD);
    	if (entry.lead == false)
	        eLead.setText ("0");
    	else
	        eLead.setText ("1");
    	element.addContent (eLead);	

	    // create <date> element and add it
    	String strDate = String.valueOf (entry.date);
	    Element eDate = new Element (XMLConstants.ELEMENT_ROUTE_DATE);
    	eDate.setText (strDate);
	    element.addContent (eDate);	

    	// create <comment> element and add it
	    Element eComment = new Element (XMLConstants.ELEMENT_ENTRY_COMMENT);
	    eComment.setText (entry.comment);
	    element.addContent (eComment);	

        // set route completed atty ("0" or "true" by default)
        String strCompleted = "1";
        if (entry.completed == false) {
            strCompleted = "0";
        }
        element.setAttribute (new Attribute (XMLConstants.ATTY_ROUTE_COMPLETED, strCompleted));
    }
}
